from .event import Event
from .event_broker import EventBroker
from .event_builders.shared.platform_events import PlatformEventsEventBuilder


class PlatformMockApi:
    def __init__(self,
                 event_broker: EventBroker,
                 sender_name: str
                 ):
        self._event_broker = event_broker
        self._sender_name = sender_name

    @property
    def event_broker(self) -> EventBroker:
        return self._event_broker

    @property
    def sender_name(self):
        return self._sender_name

    def create_hello_platform_event(self, description: str) -> Event:
        return PlatformEventsEventBuilder.hello_platform(self.sender_name, description)

    def create_goodbye_platform_event(self) -> Event:
        return PlatformEventsEventBuilder.goodbye_platform(self.sender_name)

    def create_alive_ping_event(self) -> Event:
        return PlatformEventsEventBuilder.alive_ping(self.sender_name)



from typing import Optional, Callable

from .event import Event
from .event_broker import EventBroker
from .platform_mock_api import PlatformMockApi


def connect(
        mqtt_address: str,
        mqtt_username: str,
        mqtt_password: str,
        sender_name: str,
        event_handler: Optional[Callable[[Event], None]] = None
) -> PlatformMockApi:
    if mqtt_address is None or len(mqtt_address) == 0:
        raise ValueError("mqtt_address must not be blank")
    if mqtt_username is None or len(mqtt_address) == 0:
        raise ValueError("mqtt_username must not be blank")
    if mqtt_password is None or len(mqtt_address) == 0:
        raise ValueError("mqtt_password must not be blank")
    if sender_name is None or len(mqtt_address) == 0:
        raise ValueError("sender_name must not be blank")

    import uuid
    client_id = sender_name + ":" + str(uuid.uuid4())
    event_broker = EventBroker(
        mqtt_address=mqtt_address,
        mqtt_username=mqtt_username,
        mqtt_password=mqtt_password,
        client_id=client_id,
        event_handler=event_handler
    )
    platform_mock_api = PlatformMockApi(event_broker, sender_name)
    return platform_mock_api



from enum import Enum
from typing import List, Optional

from enstadtpfaff_platform_mock_api.event import Event
from enstadtpfaff_platform_mock_api.util.topic_util import name_for_shared_topic


class AppointmentType(Enum):
    PERSONAL = 'PERSONAL'
    BUSINESS = 'BUSINESS'
    FREETIME = 'FREETIME'
    MISC = 'MISC'


# noinspection PyPep8Naming
class Appointment:
    def __init__(
            self,
            appointmentId: str,
            calendarId: str,
            calendarOwnerId: str,
            title: str,
            start: str,
            end: str,
            appointmentType: AppointmentType,
            description: Optional[str] = None,
            attendees: Optional[List[str]] = None,
            isAllDay: Optional[bool] = None,
            isPrivate: Optional[bool] = None,
            location: Optional[str] = None,
            tags: Optional[List[str]] = None
    ):
        self.appointmentId = appointmentId
        self.calendarId = calendarId
        self.calendarOwnerId = calendarOwnerId
        self.title = title
        self.start = start
        self.end = end
        self.appointmentType = None if appointmentType is None else appointmentType.value
        self.description = description
        self.attendees = attendees
        self.isAllDay = isAllDay
        self.isPrivate = isPrivate
        self.location = location
        self.tags = tags


class AppointmentsEventBuilder:
    SHARED_TOPIC_ID = "appointments"
    APPOINTMENT_CREATED_TOPIC = name_for_shared_topic(
        SHARED_TOPIC_ID,
        "appointment-created"
    )
    APPOINTMENT_UPDATED_TOPIC = name_for_shared_topic(
        SHARED_TOPIC_ID,
        "appointment-updated"
    )
    APPOINTMENT_DELETED_TOPIC = name_for_shared_topic(
        SHARED_TOPIC_ID,
        "appointment-deleted"
    )

    @staticmethod
    def appointment_created(
            appointment: Appointment
    ) -> Event:
        payload = AppointmentsEventBuilder._del_none(vars(appointment))
        return Event(
            AppointmentsEventBuilder.APPOINTMENT_CREATED_TOPIC,
            payload
        )

    @staticmethod
    def appointment_updated(
            appointment: Appointment
    ) -> Event:
        payload = AppointmentsEventBuilder._del_none(vars(appointment))
        return Event(
            AppointmentsEventBuilder.APPOINTMENT_UPDATED_TOPIC,
            payload
        )

    @staticmethod
    def appointment_deleted(
            appointment: Appointment
    ) -> Event:
        payload = AppointmentsEventBuilder._del_none(vars(appointment))
        return Event(
            AppointmentsEventBuilder.APPOINTMENT_DELETED_TOPIC,
            payload
        )

    @staticmethod
    def _del_none(d):
        """
        Delete keys with the value ``None`` in a dictionary, recursively.
        """
        if not isinstance(d, dict):
            return d
        return dict((k, AppointmentsEventBuilder._del_none(v)) for k, v in d.items() if v is not None)



from .constants import TOPIC_LEVEL_SEPARATOR, TOPIC_ROOT
from .topic_type import TopicType


def _combine_topic_paths(*path_segments: str) -> str:
    return TOPIC_LEVEL_SEPARATOR.join(path_segments)


def _typed_topic(topic_type: TopicType, topic_id: str, *additional_segments: str):
    return _combine_topic_paths(*[TOPIC_ROOT, topic_type.value, topic_id, *additional_segments])


def name_for_shared_topic(shared_topic_id: str, *additional_segments: str):
    return _typed_topic(TopicType.SHARED, shared_topic_id, *additional_segments)


def name_for_service_specific_topic(service_id: str, *additional_segments: str):
    return _typed_topic(TopicType.SERVICE_SPECIFIC, service_id, *additional_segments)



from enstadtpfaff_platform_mock_api.event import Event
from enstadtpfaff_platform_mock_api.util.topic_util import name_for_shared_topic


class PlatformEventsEventBuilder:
    SHARED_TOPIC_ID = "platform-events"
    HELLO_PLATFORM_TOPIC = name_for_shared_topic(SHARED_TOPIC_ID, "hello-platform")
    GOODBYE_PLATFORM_TOPIC = name_for_shared_topic(SHARED_TOPIC_ID, "goodbye-platform")
    ALIVE_PING_TOPIC = name_for_shared_topic(SHARED_TOPIC_ID, "alive-ping")

    @staticmethod
    def hello_platform(name: str, description: str) -> Event:
        return Event(
            topic=PlatformEventsEventBuilder.HELLO_PLATFORM_TOPIC,
            payload={
                'name': name,
                'description': description
            }
        )

    @staticmethod
    def goodbye_platform(name: str) -> Event:
        return Event(
            topic=PlatformEventsEventBuilder.GOODBYE_PLATFORM_TOPIC,
            payload={
                'name': name
            }
        )

    @staticmethod
    def alive_ping(name: str) -> Event:
        return Event(
            topic=PlatformEventsEventBuilder.ALIVE_PING_TOPIC,
            payload={
                'name': name
            }
        )



__version__ = '0.5.0'

from .event import Event
from .platform_mock_api_factory import connect
from .util.topic_util import name_for_service_specific_topic
from .util.topic_util import name_for_shared_topic



import json


class Event:
    def __init__(self, topic: str, payload):
        self._topic: str = topic
        if isinstance(payload, str):
            self._payload: str = payload
        else:
            self._payload: str = json.dumps(payload)

    def __str__(self):
        return "Event{" + \
               "topic='" + self.topic + '\'' + \
               ", payload='" + self.payload + '\'' + \
               '}'

    @property
    def topic(self) -> str:
        return self._topic

    @property
    def payload(self) -> str:
        return self._payload



from enum import Enum
from typing import List, Optional

from enstadtpfaff_platform_mock_api.event import Event
from enstadtpfaff_platform_mock_api.util.topic_util import name_for_shared_topic


class Category(Enum):
    NATURE = 'NATURE'
    HOUSEHOLD = 'HOUSEHOLD'
    MOBILITY = 'MOBILITY'
    ENERGY = 'ENERGY'
    FAMILY = 'FAMILY'
    GROCERIES = 'GROCERIES'
    FREETIME = 'FREETIME'
    HEATING = 'HEATING'
    MISC = 'MISC'


# noinspection PyPep8Naming
class Hint:
    def __init__(
            self,
            headline: str,
            content: str,
            categories: List[Category],
            author: Optional[str] = None,
            tags: Optional[List[str]] = None,
            publishedBy: Optional[str] = None,
            publishedAt: Optional[str] = None,
            basedOn: Optional[List[str]] = None,
            validityBegin: Optional[str] = None,
            validityEnd: Optional[str] = None
    ):
        self.headline = headline
        self.content = content
        self.categories = list(
            map((lambda category: category if isinstance(category, str) else category.value), categories)
        )
        self.author = author
        self.tags = tags
        self.publishedBy = publishedBy
        self.publishedAt = publishedAt
        self.basedOn = basedOn
        self.validityBegin = validityBegin
        self.validityEnd = validityEnd


class HintsEventBuilder:
    SHARED_TOPIC_ID = "hints"
    HINTS_TOPIC = name_for_shared_topic(
        SHARED_TOPIC_ID
    )

    @staticmethod
    def hint(
            hint: Hint
    ) -> Event:
        payload = HintsEventBuilder._del_none(vars(hint))
        return Event(
            HintsEventBuilder.HINTS_TOPIC,
            payload
        )

    @staticmethod
    def _del_none(d):
        """
        Delete keys with the value ``None`` in a dictionary, recursively.
        """
        if not isinstance(d, dict):
            return d
        return dict((k, HintsEventBuilder._del_none(v)) for k, v in d.items() if v is not None)



from enstadtpfaff_platform_mock_api.event import Event
from enstadtpfaff_platform_mock_api.util.topic_util import name_for_service_specific_topic


class EchoServiceEventBuilder:
    SERVICE_ID = 'pm-echo-service'
    PING_TOPIC = name_for_service_specific_topic(SERVICE_ID, 'ping')

    @staticmethod
    def ping(payload: str) -> Event:
        return Event(EchoServiceEventBuilder.PING_TOPIC, payload)



import re
from enum import Enum
from typing import Optional

from enstadtpfaff_platform_mock_api.event import Event
from enstadtpfaff_platform_mock_api.util.topic_util import name_for_shared_topic


class Category(Enum):
    INDOOR_CLIMATE = "indoor-climate"
    LIGHTING = "lighting"
    RADIATION = "radiation"
    WINDOW = "window"
    UNIT_DOOR = "unit-door"
    BALCONY_DOOR = "balcony-door"
    ELECTRICITY = "electricity"
    PRESENCE = "presence"
    BLIND = "blind"
    GATE = "gate"
    SOCKET = "socket"
    MISC = "misc"
    MAILBOX = "mailbox"
    DOORBELL = "doorbell"


class ThermostatMode(Enum):
    OFF = 'OFF'
    ON = 'ON'
    HEAT = 'HEAT'
    COOL = 'COOL'
    AUTO = 'AUTO'


# noinspection PyPep8Naming
class IndoorClimate:
    def __init__(
            self,
            currentTemperature: Optional[float] = None,
            targetTemperature: Optional[float] = None,
            relativeHumidity: Optional[int] = None,
            pressure: Optional[float] = None,
            thermostatMode: Optional[ThermostatMode] = None,
            currentHeatingPower: Optional[float] = None,
            currentCoolingPower: Optional[float] = None
    ):
        self.currentTemperature = currentTemperature
        self.targetTemperature = targetTemperature
        self.relativeHumidity = relativeHumidity
        self.pressure = pressure
        self.thermostatMode = None if thermostatMode is None else thermostatMode.value
        self.currentHeatingPower = currentHeatingPower
        self.currentCoolingPower = currentCoolingPower


class OnOffState(Enum):
    OFF = "OFF"
    ON = "ON"


# noinspection PyPep8Naming
class Lighting:
    def __init__(
            self,
            dimmingLevel: Optional[int] = None,
            color: Optional[tuple] = None,
            colorTemperature: Optional[float] = None,
            state: Optional[OnOffState] = None
    ):
        if color is not None:
            if len(color) != 4:
                raise ValueError("invalid color")
            if color[0] < 0 or color[0] > 255:
                raise ValueError("invalid color")
            if color[1] < 0 or color[1] > 255:
                raise ValueError("invalid color")
            if color[2] < 0 or color[2] > 255:
                raise ValueError("invalid color")
            if color[3] < 0 or color[3] > 255:
                raise ValueError("invalid color")
        self.dimmingLevel = dimmingLevel
        self.color = color
        self.colorTemperature = colorTemperature
        self.state = None if state is None else state.value


# noinspection PyPep8Naming
class Radiation:
    def __init__(
            self,
            uvIndex: Optional[int] = None,
            brightness: Optional[float] = None
    ):
        self.uvIndex = uvIndex
        self.brightness = brightness


class OpeningState(Enum):
    LOCKED = "LOCKED"
    CLOSED = "CLOSED"
    OPEN = "OPEN"
    TILTED = "TILTED"


# noinspection PyPep8Naming
class Window:
    def __init__(
            self,
            openingState: Optional[OpeningState] = None,
    ):
        self.openingState = None if openingState is None else openingState.value


# noinspection PyPep8Naming
class UnitDoor:
    def __init__(
            self,
            openingState: Optional[OpeningState] = None
    ):
        self.openingState = None if openingState is None else openingState.value


# noinspection PyPep8Naming
class BalconyDoor:
    def __init__(
            self,
            openingState: Optional[OpeningState] = None
    ):
        self.openingState = None if openingState is None else openingState.value


# noinspection PyPep8Naming
class Electricity:
    def __init__(
            self,
            currentPower: Optional[float] = None,
            current: Optional[float] = None,
            voltage: Optional[float] = None
    ):
        self.currentPower = currentPower
        self.current = current
        self.voltage = voltage


# noinspection PyPep8Naming
class Presence:
    def __init__(
            self,
            presenceDetected: Optional[bool] = None,
            presenceAmount: Optional[int] = None
    ):
        self.presenceDetected = presenceDetected
        self.presenceAmount = presenceAmount


class MovementMode(Enum):
    OPENING = "OPENING"
    CLOSING = "CLOSING"
    IDLE = "IDLE"


# noinspection PyPep8Naming
class Blind:
    def __init__(
            self,
            blindLevel: Optional[int] = None,
            movementMode: Optional[MovementMode] = None,
            blindAngle: Optional[float] = None
    ):
        self.blindLevel = blindLevel
        self.movementMode = None if movementMode is None else movementMode.value
        self.blindAngle = blindAngle


# noinspection PyPep8Naming
class Gate:
    def __init__(
            self,
            openingState: Optional[OpeningState] = None,
            movementMode: Optional[MovementMode] = None,
    ):
        self.openingState = None if openingState is None else openingState.value
        self.movementMode = None if movementMode is None else movementMode.value


class Socket:
    def __init__(
            self,
            state: Optional[OnOffState] = None
    ):
        self.state = None if state is None else state.value


class Misc:
    def __init__(
            self,
            state: Optional[OnOffState] = None
    ):
        self.state = None if state is None else state.value


# noinspection PyPep8Naming
class Mailbox:
    def __init__(
            self,
            contentAvailable: Optional[bool] = None,
            contentCount: Optional[int] = None
    ):
        self.contentAvailable = contentAvailable
        self.contentCount = contentCount


# noinspection PyPep8Naming
class Doorbell:
    def __init__(
            self,
            ringTime: Optional[str] = None,
    ):
        self.ringTime = ringTime


class SmartHomeEventBuilder:
    # TODO apply some more validation...
    SHARED_TOPIC_ID = "smart-home"
    SMART_HOME_TOPIC_LAYOUT = name_for_shared_topic(
        SHARED_TOPIC_ID,
        "{}",  # house-identifier
        "{}",  # floor-identifier
        "{}",  # unit-identifier
        "{}",  # room-identifier
        "{}",  # category-identifier
        "{}"  # device-identifier
    )
    TOPIC_SEGMENT_PATTERN = re.compile("^[a-z0-9-.]+$")

    @staticmethod
    def indoor_climate(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            indoor_climate: IndoorClimate
    ) -> Event:
        payload = {
            'currentTemperature': indoor_climate.currentTemperature,
            'targetTemperature': indoor_climate.targetTemperature,
            'relativeHumidity': indoor_climate.relativeHumidity,
            'pressure': indoor_climate.pressure,
            'thermostatMode': indoor_climate.thermostatMode,
            'currentHeatingPower': indoor_climate.currentHeatingPower,
            'currentCoolingPower': indoor_climate.currentCoolingPower
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.INDOOR_CLIMATE, payload)

    @staticmethod
    def lighting(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            lighting: Lighting
    ) -> Event:
        payload = {
            'dimmingLevel': lighting.dimmingLevel,
            'color': lighting.color,
            'colorTemperature': lighting.colorTemperature,
            'state': lighting.state
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.LIGHTING, payload)

    @staticmethod
    def radiation(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            radiation: Radiation
    ) -> Event:
        payload = {
            'uvIndex': radiation.uvIndex,
            'brightness': radiation.brightness
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.RADIATION, payload)

    @staticmethod
    def window(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            window: Window
    ) -> Event:
        payload = {
            'openingState': window.openingState
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.WINDOW, payload)

    @staticmethod
    def unit_door(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            unit_door: UnitDoor
    ) -> Event:
        payload = {
            'openingState': unit_door.openingState
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.UNIT_DOOR, payload)

    @staticmethod
    def balcony_door(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            balcony_door: BalconyDoor
    ) -> Event:
        payload = {
            'openingState': balcony_door.openingState
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.BALCONY_DOOR, payload)

    @staticmethod
    def electricity(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            electricity: Electricity
    ) -> Event:
        payload = {
            'currentPower': electricity.currentPower,
            'current': electricity.current,
            'voltage': electricity.voltage
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.ELECTRICITY, payload)

    @staticmethod
    def presence(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            presence: Presence
    ) -> Event:
        payload = {
            'presenceDetected': presence.presenceDetected,
            'presenceAmount': presence.presenceAmount
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.PRESENCE, payload)

    @staticmethod
    def blind(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            blind: Blind
    ) -> Event:
        payload = {
            'blindLevel': blind.blindLevel,
            'movementMode': blind.movementMode,
            'blindAngle': blind.blindAngle
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.BLIND, payload)

    @staticmethod
    def gate(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            gate: Gate
    ) -> Event:
        payload = {
            'openingState': gate.openingState,
            'movementMode': gate.movementMode
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.GATE, payload)

    @staticmethod
    def socket(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            socket: Socket
    ) -> Event:
        payload = {
            'state': socket.state
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.SOCKET, payload)

    @staticmethod
    def misc(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            misc: Misc
    ) -> Event:
        payload = {
            'state': misc.state
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.MISC, payload)

    @staticmethod
    def mailbox(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            mailbox: Mailbox
    ) -> Event:
        payload = {
            'contentAvailable': mailbox.contentAvailable,
            'contentCount': mailbox.contentCount
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.MAILBOX, payload)

    @staticmethod
    def doorbell(
            house_identifier: str,
            floor_identifier: str,
            unit_identifier: str,
            room_identifier: str,
            device_identifier: str,
            doorbell: Doorbell
    ) -> Event:
        payload = {
            'ringTime': doorbell.ringTime
        }
        return SmartHomeEventBuilder._build_for_category(house_identifier, floor_identifier, unit_identifier,
                                                         room_identifier,
                                                         device_identifier, Category.DOORBELL, payload)

    @staticmethod
    def _build_for_category(house_identifier: str,
                            floor_identifier: str,
                            unit_identifier: str,
                            room_identifier: str,
                            device_identifier: str,
                            category: Category,
                            payload: dict) -> Event:
        if not SmartHomeEventBuilder.TOPIC_SEGMENT_PATTERN.match(house_identifier):
            raise ValueError("invalid house_identifier")
        if not SmartHomeEventBuilder.TOPIC_SEGMENT_PATTERN.match(floor_identifier):
            raise ValueError("invalid floor_identifier")
        if not SmartHomeEventBuilder.TOPIC_SEGMENT_PATTERN.match(unit_identifier):
            raise ValueError("invalid unit_identifier")
        if not SmartHomeEventBuilder.TOPIC_SEGMENT_PATTERN.match(room_identifier):
            raise ValueError("invalid room_identifier")
        if not SmartHomeEventBuilder.TOPIC_SEGMENT_PATTERN.match(device_identifier):
            raise ValueError("invalid device_identifier")
        if (
                (category is None) or
                (not isinstance(category, Category)) or
                (not SmartHomeEventBuilder.TOPIC_SEGMENT_PATTERN.match(category.value))
        ):
            raise ValueError("invalid category")

        if payload is None:
            raise ValueError("invalid payload")

        payload = SmartHomeEventBuilder._del_none(payload)  # get rid of None/nulls

        return Event(
            topic=SmartHomeEventBuilder.SMART_HOME_TOPIC_LAYOUT.format(
                house_identifier,
                floor_identifier,
                unit_identifier,
                room_identifier,
                category.value,
                device_identifier
            ),
            payload=payload
        )

    @staticmethod
    def _del_none(d):
        """
        Delete keys with the value ``None`` in a dictionary, recursively.
        """
        if not isinstance(d, dict):
            return d
        return dict((k, SmartHomeEventBuilder._del_none(v)) for k, v in d.items() if v is not None)



from typing import Optional, Callable

from paho.mqtt.client import Client, MQTTMessage

from .event import Event


class EventBroker:
    def __init__(self,
                 mqtt_address: str,
                 mqtt_username: str,
                 mqtt_password: str,
                 client_id: str,
                 event_handler: Optional[Callable[[Event], None]] = None):
        self._client = Client(client_id=client_id, clean_session=True)
        self._client.username_pw_set(username=mqtt_username, password=mqtt_password)
        self._client.tls_set()
        mqtt_address = mqtt_address.replace('ssl://', '')
        [host_str, port_str] = mqtt_address.split(':')
        self._client.connect(host=host_str, port=int(port_str))
        self._event_handler = event_handler
        if self._event_handler is not None:
            self._client.message_callback_add('#', self._on_message)

        # TODO reconnect + what about the subscriptions?
        self._client.loop_start()

    def _on_message(self, client: Client, userdata, message: MQTTMessage):
        assert self._event_handler is not None
        event = Event(message.topic, message.payload.decode("utf8"))
        self._event_handler(event)

    def subscribe(self, topic: str) -> None:
        if self._event_handler is not None:
            self._client.subscribe(topic=topic)  # TODO check reconnect resubscribe...
        else:
            print("event_handler is None, subscribe = noop")

    def unsubscribe(self, topic: str) -> None:
        if self._event_handler is not None:
            self._client.unsubscribe(topic)
        else:
            print("event_handler is None, unsubscribe = noop")

    def disconnect(self) -> None:
        self._client.disconnect()

    def publish(self, event: Event) -> None:
        self._client.publish(event.topic, event.payload)



import codecs
import os.path

from setuptools import setup, find_packages


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


setup(
    name='enstadtpfaff-platform-mock-api',
    version=get_version('enstadtpfaff_platform_mock_api/__init__.py'),
    packages=find_packages(exclude=('enstadtpfaff_platform_mock_api_playground',),
                           include=('enstadtpfaff_platform_mock_api', 'enstadtpfaff_platform_mock_api.*',)),
    url='',
    license='',
    author='Platform Development Team',
    author_email='platform-dev@iese.fraunhofer.de',
    description='This is the Python API for the EnStadt:Pfaff Platform Mock',
    install_requires=[
        'paho-mqtt'
    ]
)



from enstadtpfaff_platform_mock_api.event import Event
from enstadtpfaff_platform_mock_api.util.topic_util import name_for_shared_topic


class ChatMessagesEventBuilder:
    SHARED_TOPIC_ID = "chat-messages"
    CHAT_MESSAGES_TOPIC = name_for_shared_topic(SHARED_TOPIC_ID)

    @staticmethod
    def chat_message(application_name: str, from_: str, to: str, message: str) -> Event:
        return Event(
            topic=ChatMessagesEventBuilder.CHAT_MESSAGES_TOPIC,
            payload={
                'senderApplication': application_name,
                'from': from_,
                'to': to,
                'message': message
            }
        )
